@@ -15,9 +15,13 @@ Here commonly used functions that do not need aiida-stuff (i.e. can be tested
 without a database) are collected.
 """
 import io
+from typing import Any, Dict, Generator, Iterable, NamedTuple, Tuple, List, TypeVar, Union
+try:
+    from typing import TypeAlias  #type:ignore
+except ImportError:
+    from typing_extensions import TypeAlias
 import numpy as np
 import warnings
-from collections import namedtuple
 ####################################################################################
 
 #helper functions used in calculation, parser etc.
@@ -29,8 +33,7 @@ def open_general(filename_or_handle, iomode=None):
     Also take care of closed files by reopenning them.
     This is intended to be used like this::
 
-        f = open_general(outfile)
-        with f: # make sure the file is properly closed
+        with open_general(outfile) as f:
             txt = f.readlines()
     """
     reopen_file = False
@@ -41,7 +44,7 @@ def open_general(filename_or_handle, iomode=None):
 
     if reopen_file:
         if iomode is None:
-            iomode = u'r'
+            iomode = 'r'
         f = open(filename_or_handle, iomode)
     else:
         f = filename_or_handle
@@ -55,7 +58,7 @@ def open_general(filename_or_handle, iomode=None):
     return f
 
 
-def skipHeader(seq, n):
+def skipHeader(seq: Iterable[Any], n: int) -> Generator[Any, None, None]:
     """Iterate over a sequence skipping the first n elements
 
     Args:
@@ -70,7 +73,7 @@ def skipHeader(seq, n):
             yield item
 
 
-def filter_out_empty_dict_entries(dict_to_filter):
+def filter_out_empty_dict_entries(dict_to_filter: Dict) -> Dict:
     """
     Filter out entries in a given dict that correspond to empty values.
     At the moment this is empty lists, dicts and None
@@ -80,7 +83,7 @@ def filter_out_empty_dict_entries(dict_to_filter):
     :returns: dict without empty entries
     """
 
-    EMPTY_VALUES = (None, [], {})
+    EMPTY_VALUES: Tuple[None, List, Dict] = (None, [], {})
 
     return {key: val for key, val in dict_to_filter.items() if val not in EMPTY_VALUES}
 
@@ -145,7 +148,7 @@ def angles_to_vec(magnitude, theta, phi):
     return vec
 
 
-def get_Ang2aBohr():
+def get_Ang2aBohr() -> float:
     from masci_tools.util.constants import ANG_BOHR_KKR
     #warnings.warn(
     #    'get_Ang2aBohr is deprecated. Use 1/BOHR_A with the BOHR_A constant from the module masci_tools.util.constants instead',
@@ -153,7 +156,7 @@ def get_Ang2aBohr():
     return ANG_BOHR_KKR
 
 
-def get_aBohr2Ang():
+def get_aBohr2Ang() -> float:
     from masci_tools.util.constants import ANG_BOHR_KKR
     #warnings.warn(
     #    'get_aBohr2Ang is deprecated. Use the BOHR_A constant from the module masci_tools.util.constants instead',
@@ -161,7 +164,7 @@ def get_aBohr2Ang():
     return 1.0 / ANG_BOHR_KKR
 
 
-def get_Ry2eV():
+def get_Ry2eV() -> float:
     from masci_tools.util.constants import RY_TO_EV_KKR
     #warnings.warn(
     #    'get_Ry2eV is deprecated. Use the RY_TO_EV constant from the module masci_tools.util.constants instead',
@@ -193,8 +196,7 @@ def vec_to_angles(vec):
 
 
 def get_version_info(outfile):
-    f = open_general(outfile)
-    with f:
+    with open_general(outfile) as f:
         tmptxt = f.readlines()
     itmp = search_string('Code version:', tmptxt)
     if itmp == -1:  # try to find serial number from header of file
@@ -213,8 +215,7 @@ def get_version_info(outfile):
 
 def get_corestates_from_potential(potfile='potential'):
     """Read core states from potential file"""
-    f = open_general(potfile)
-    with f:
+    with open_general(potfile) as f:
         txt = f.readlines()
 
     #get start of each potential part
@@ -246,7 +247,7 @@ def get_highest_core_state(nstates, energies, lmoments):
     idx = energies.argmax()
     lval = lmoments[idx]
     nquant = sum(lmoments == lval) + lval
-    level_descr = '%i%s' % (nquant, 'spdfgh'[lval])
+    level_descr = f"{nquant}{'spdfgh'[lval]}"
 
     return lval, energies[idx], level_descr
 
@@ -359,7 +360,7 @@ def get_ef_from_potfile(potfile):
     return ef
 
 
-def convert_to_pystd(value):
+def convert_to_pystd(value: Any) -> Any:
     """Recursively convert numpy datatypes to standard python, needed by aiida-core.
 
     Usage:
@@ -387,7 +388,7 @@ def convert_to_pystd(value):
     return value
 
 
-def camel_to_snake(name):
+def camel_to_snake(name: str) -> str:
     """
     Converts camelCase to snake_case variable names
     Used in the Fleur parser to convert attribute names from the xml files
@@ -396,7 +397,7 @@ def camel_to_snake(name):
     return ''.join(['_' + c.lower() if c.isupper() else c for c in name]).lstrip('_')
 
 
-def convert_to_fortran(val, quote_strings=True):
+def convert_to_fortran(val: Any, quote_strings: bool = True) -> str:
     """
     :param val: the value to be read and converted to a Fortran-friendly string.
     """
@@ -425,7 +426,7 @@ def convert_to_fortran(val, quote_strings=True):
     return val_str
 
 
-def convert_to_fortran_string(string):
+def convert_to_fortran_string(string: str) -> str:
     """
     converts some parameter strings to the format for the inpgen
     :param string: some string
@@ -434,25 +435,23 @@ def convert_to_fortran_string(string):
     if not string.strip().startswith("\"") or \
        not string.strip().endswith("\""):
         return f'"{string}"'
-    else:
-        return string
+    return string
 
 
-def is_sequence(arg):
+def is_sequence(arg: Any) -> bool:
     """
     Checks if arg is a sequence
     """
-    if isinstance(arg, str):
-        return False
-    elif hasattr(arg, '__iter__'):
-        return True
-    elif not hasattr(arg, 'strip') and hasattr(arg, '__getitem__'):
-        return True
-    else:
-        return False
+    from collections.abc import Sequence
+    return isinstance(arg, Sequence) and not isinstance(arg, str)
 
 
-def abs_to_rel(vector, cell):
+_TVectorType = TypeVar('_TVectorType', Tuple[float, float, float], List[float], np.ndarray)
+"""Generic type variable for atom position types"""
+VectorType: TypeAlias = Union[Tuple[float, float, float], List[float], np.ndarray]
+
+
+def abs_to_rel(vector: _TVectorType, cell: Union[List[List[float]], np.ndarray]) -> _TVectorType:
     """
     Converts a position vector in absolute coordinates to relative coordinates.
 
@@ -460,20 +459,28 @@ def abs_to_rel(vector, cell):
     :param cell: Bravais matrix of a crystal 3x3 Array, List of list or np.array
     :return: list of length 3 of scaled vector, or False if vector was not length 3
     """
-
-    if len(vector) == 3:
-        cell_np = np.array(cell)
-        inv_cell_np = np.linalg.inv(cell_np)
-        postionR = np.array(vector)
-        # np.matmul(inv_cell_np, postionR)#
-        new_rel_post = np.matmul(postionR, inv_cell_np)
-        new_rel_pos = list(new_rel_post)
-        return new_rel_pos
+    if not isinstance(vector, np.ndarray):
+        vector_np = np.array(vector)
     else:
-        return False
+        vector_np = vector
+
+    if not isinstance(cell, np.ndarray):
+        cell = np.array(cell)
+
+    if len(vector_np) != 3:
+        raise ValueError('Vector must be of length 3')
+
+    relative_vector = vector_np @ np.linalg.inv(cell)
+
+    if isinstance(vector, list):
+        return relative_vector.tolist()
+    if isinstance(vector, tuple):
+        return tuple(relative_vector)  #type:ignore
+    return relative_vector
 
 
-def abs_to_rel_f(vector, cell, pbc):
+def abs_to_rel_f(vector: _TVectorType, cell: Union[List[List[float]], np.ndarray], pbc: Tuple[bool, bool,
+                                                                                              bool]) -> _TVectorType:
     """
     Converts a position vector in absolute coordinates to relative coordinates
     for a film system.
@@ -486,26 +493,31 @@ def abs_to_rel_f(vector, cell, pbc):
     # TODO this currently only works if the z-coordinate is the one with no pbc
     # Therefore if a structure with x non pbc is given this should also work.
     # maybe write a 'tranform film to fleur_film routine'?
-    if len(vector) == 3:
-        if not pbc[2]:
-            # leave z coordinate absolute
-            # convert only x and y.
-            postionR = np.array(vector)
-            postionR_f = np.array(postionR[:2])
-            cell_np = np.array(cell)
-            cell_np = np.array(cell_np[0:2, 0:2])
-            inv_cell_np = np.linalg.inv(cell_np)
-            # np.matmul(inv_cell_np, postionR_f)]
-            new_xy = np.matmul(postionR_f, inv_cell_np)
-            new_rel_pos_f = [new_xy[0], new_xy[1], postionR[2]]
-            return new_rel_pos_f
-        else:
-            print('FLEUR can not handle this type of film coordinate')
+    if not isinstance(vector, np.ndarray):
+        vector_np = np.array(vector)
     else:
-        return False
+        vector_np = vector
+
+    if not isinstance(cell, np.ndarray):
+        cell = np.array(cell)
+
+    if len(vector_np) != 3:
+        raise ValueError('Vector must be of length 3')
+
+    if pbc[2]:
+        raise ValueError('FLEUR can not handle this type of film coordinate')
+
+    relative_vector_f = vector_np[:2] @ np.linalg.inv(cell[0:2, 0:2])
+    relative_vector = np.append(relative_vector_f, vector_np[2])
+
+    if isinstance(vector, list):
+        return relative_vector.tolist()
+    if isinstance(vector, tuple):
+        return tuple(relative_vector)  #type:ignore
+    return relative_vector
 
 
-def rel_to_abs(vector, cell):
+def rel_to_abs(vector: _TVectorType, cell: Union[List[List[float]], np.ndarray]) -> _TVectorType:
     """
     Converts a position vector in internal coordinates to absolute coordinates
     in Angstrom.
@@ -514,17 +526,27 @@ def rel_to_abs(vector, cell):
     :param cell: Bravais matrix of a crystal 3x3 Array, List of list or np.array
     :return: list of legth 3 of scaled vector, or False if vector was not lenth 3
     """
-    if len(vector) == 3:
-        cell_np = np.array(cell)
-        postionR = np.array(vector)
-        new_abs_post = np.matmul(postionR, cell_np)
-        new_abs_pos = list(new_abs_post)
-        return new_abs_pos
+    if not isinstance(vector, np.ndarray):
+        vector_np = np.array(vector)
     else:
-        return False
+        vector_np = vector
+
+    if not isinstance(cell, np.ndarray):
+        cell = np.array(cell)
+
+    if len(vector_np) != 3:
+        raise ValueError('Vector must be of length 3')
+
+    absolute_vector = vector_np @ cell
+
+    if isinstance(vector, list):
+        return absolute_vector.tolist()
+    if isinstance(vector, tuple):
+        return tuple(absolute_vector)  #type:ignore
+    return absolute_vector
 
 
-def rel_to_abs_f(vector, cell):
+def rel_to_abs_f(vector: _TVectorType, cell: Union[List[List[float]], np.ndarray]) -> _TVectorType:
     """
     Converts a position vector in internal coordinates to absolute coordinates
     in Angstrom for a film structure (2D).
@@ -532,19 +554,34 @@ def rel_to_abs_f(vector, cell):
     # TODO this currently only works if the z-coordinate is the one with no pbc
     # Therefore if a structure with x non pbc is given this should also work.
     # maybe write a 'tranform film to fleur_film routine'?
-    if len(vector) == 3:
-        postionR = np.array(vector)
-        postionR_f = np.array(postionR[:2])
-        cell_np = np.array(cell)
-        cell_np = np.array(cell_np[0:2, 0:2])
-        new_xy = np.matmul(postionR_f, cell_np)
-        new_abs_pos_f = [new_xy[0], new_xy[1], postionR[2]]
-        return new_abs_pos_f
+    if not isinstance(vector, np.ndarray):
+        vector_np = np.array(vector)
     else:
-        return False
+        vector_np = vector
+
+    if not isinstance(cell, np.ndarray):
+        cell = np.array(cell)
+
+    if len(vector_np) != 3:
+        raise ValueError('Vector must be of length 3')
+
+    absolute_vector_f = vector_np[:2] @ cell[0:2, 0:2]
+    absolute_vector: np.ndarray = np.append(absolute_vector_f, vector_np[2])
+
+    if isinstance(vector, list):
+        return absolute_vector.tolist()
+    if isinstance(vector, tuple):
+        return tuple(absolute_vector)  #type:ignore
+    return absolute_vector  #type:ignore
 
 
-def find_symmetry_relation(from_pos, to_pos, rotations, shifts, cell, relative_pos=False, film=False):
+def find_symmetry_relation(from_pos: VectorType,
+                           to_pos: VectorType,
+                           rotations: List[np.ndarray],
+                           shifts: List[np.ndarray],
+                           cell: Union[List[List[float]], np.ndarray],
+                           relative_pos: bool = False,
+                           film: bool = False) -> Tuple[np.ndarray, np.ndarray]:
     """
     Find symmetry relation between the given vectors. This functions assumes
     that a symmetry relation exists otherwise an error is raised
@@ -562,7 +599,7 @@ def find_symmetry_relation(from_pos, to_pos, rotations, shifts, cell, relative_p
     :raises ValueError: If no symmetry relation is found
     """
 
-    def lattice_shifts():
+    def lattice_shifts() -> Generator[np.ndarray, None, None]:
         for i in range(-2, 3):
             for j in range(-2, 3):
                 for k in range(-2, 3):
@@ -590,11 +627,16 @@ def find_symmetry_relation(from_pos, to_pos, rotations, shifts, cell, relative_p
     raise ValueError(f'No symmetry relation found between {from_pos} and {to_pos}')
 
 
-#namedtuple used for input output of atom sites
-AtomSiteProperties = namedtuple('AtomSiteProperties', ('position', 'symbol', 'kind'))
+class AtomSiteProperties(NamedTuple):
+    """
+    namedtuple used for input output of atom sites
+    """
+    position: List[float]  #TODO could be made generic with VectorType
+    symbol: str
+    kind: str
 
 
-def get_wigner_matrix(l, phi, theta):
+def get_wigner_matrix(l: int, phi: float, theta: float) -> np.ndarray:
     """Produces the wigner rotation matrix for the density matrix
 
     :param l: int, orbital quantum number
@@ -616,9 +658,8 @@ def get_wigner_matrix(l, phi, theta):
     return d_wigner
 
 
-def fac(n):
+def fac(n: int) -> int:
     """Returns the factorial of n"""
     if n < 2:
         return 1
-    else:
-        return n * fac(n - 1)
+    return n * fac(n - 1)
