@@ -20,10 +20,12 @@ from pprint import pprint
 from masci_tools.util.constants import HTR_TO_EV
 from masci_tools.util.parse_tasks_decorators import conversion_function
 from masci_tools.io.common_functions import convert_to_pystd
+from typing import Dict, Any
+from logging import Logger
 
 
 @conversion_function
-def convert_total_energy(out_dict, logger):
+def convert_total_energy(out_dict: Dict[str, Any], logger: Logger) -> Dict[str, Any]:
     """
     Convert total energy to eV
     """
@@ -55,7 +57,7 @@ def convert_total_energy(out_dict, logger):
 
 
 @conversion_function
-def calculate_total_magnetic_moment(out_dict, logger):
+def calculate_total_magnetic_moment(out_dict: Dict[str, Any], logger: Logger) -> Dict[str, Any]:
     """
     Calculate the the total magnetic moment per cell
 
@@ -79,7 +81,7 @@ def calculate_total_magnetic_moment(out_dict, logger):
 
 
 @conversion_function
-def calculate_walltime(out_dict, logger):
+def calculate_walltime(out_dict: Dict[str, Any], logger: Logger) -> Dict[str, Any]:
     """
     Calculate the walltime from start and end time
 
@@ -138,7 +140,7 @@ def calculate_walltime(out_dict, logger):
 
 
 @conversion_function
-def convert_ldau_definitions(out_dict, logger):
+def convert_ldau_definitions(out_dict: Dict[str, Any], logger: Logger) -> Dict[str, Any]:
     """
     Convert the parsed information from LDA+U into a more readable dict
 
@@ -182,7 +184,7 @@ def convert_ldau_definitions(out_dict, logger):
 
 
 @conversion_function
-def convert_relax_info(out_dict, logger):
+def convert_relax_info(out_dict: Dict[str, Any], logger: Logger) -> Dict[str, Any]:
     """
     Convert the general relaxation information
 
@@ -210,7 +212,7 @@ def convert_relax_info(out_dict, logger):
 
 
 @conversion_function
-def convert_forces(out_dict, logger):
+def convert_forces(out_dict: Dict[str, Any], logger: Logger) -> Dict[str, Any]:
     """
     Convert the parsed forces from a iteration
 
