@@ -14,19 +14,26 @@
 Functions for modifying the xml input file of Fleur with explicit xpath arguments
 These can still use the schema dict for finding information about the xpath
 """
+from typing import Any, Iterable, List, Union, Dict, cast, Tuple
+try:
+    from typing import Literal
+except ImportError:
+    from typing_extensions import Literal  #type:ignore
 from lxml import etree
 from masci_tools.util.xml.common_functions import eval_xpath
-
+from masci_tools.io.parsers import fleur_schema
 ######################CREATING/DELETING TAGS###############################################
 
 
-def xml_create_tag_schema_dict(xmltree,
-                               schema_dict,
-                               xpath,
-                               base_xpath,
-                               element,
-                               create_parents=False,
-                               occurrences=None):
+def xml_create_tag_schema_dict(
+        xmltree: Union[etree._Element, etree._ElementTree],
+        schema_dict: 'fleur_schema.SchemaDict',
+        xpath: 'etree._xpath',
+        base_xpath: str,
+        element: Union[str, etree._Element],
+        create_parents: bool = False,
+        number_nodes: int = 1,
+        occurrences: Union[int, Iterable[int]] = None) -> Union[etree._Element, etree._ElementTree]:
     """
     This method evaluates an xpath expression and creates a tag in a xmltree under the
     returned nodes.
@@ -43,6 +50,7 @@ def xml_create_tag_schema_dict(xmltree,
                            the parent tags are created recursively
     :param occurrences: int or list of int. Which occurence of the parent nodes to create a tag.
                         By default all nodes are used.
+    :param number_nodes: how many identical nodes to create
 
     :raises ValueError: If the nodes are missing and `create_parents=False`
 
@@ -63,7 +71,7 @@ def xml_create_tag_schema_dict(xmltree,
         except ValueError as exc:
             raise ValueError(f"Failed to construct etree Element from '{element_name}'") from exc
     else:
-        element_name = element.tag
+        element_name = element.tag  #type:ignore
 
     if len(tag_info['order']) == 0:
         tag_order = None
@@ -72,12 +80,15 @@ def xml_create_tag_schema_dict(xmltree,
 
     several_tags = element_name in tag_info['several']
 
-    parent_nodes = eval_xpath(xmltree, xpath, list_return=True)
+    if not several_tags and number_nodes > 1:
+        raise ValueError(f'Can not create {number_nodes} nodes of tag {element_name}: Tag only allowed once')
+
+    parent_nodes: List[etree._Element] = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
 
     if len(parent_nodes) == 0:
         if create_parents:
             parent_xpath, parent_name = split_off_tag(base_xpath)
-            complex_parent_xpath, _ = split_off_tag(xpath)
+            complex_parent_xpath, _ = split_off_tag(str(xpath))
             xmltree = xml_create_tag_schema_dict(xmltree,
                                                  schema_dict,
                                                  complex_parent_xpath,
@@ -88,16 +99,19 @@ def xml_create_tag_schema_dict(xmltree,
             raise ValueError(f"Could not create tag '{element_name}' because atleast one subtag is missing. "
                              'Use create=True to create the subtags')
 
-    return xml_create_tag(xmltree, xpath, element, tag_order=tag_order, occurrences=occurrences, several=several_tags)
+    for _ in range(number_nodes):
+        xml_create_tag(xmltree, xpath, element, tag_order=tag_order, occurrences=occurrences, several=several_tags)
+    return xmltree
 
 
-def eval_xpath_create(xmltree,
-                      schema_dict,
-                      xpath,
-                      base_xpath,
-                      create_parents=False,
-                      occurrences=None,
-                      list_return=False):
+def eval_xpath_create(xmltree: Union[etree._Element, etree._ElementTree],
+                      schema_dict: 'fleur_schema.SchemaDict',
+                      xpath: 'etree._xpath',
+                      base_xpath: str,
+                      create_parents: bool = False,
+                      occurrences: Union[int, Iterable[int]] = None,
+                      number_nodes: int = 1,
+                      list_return: bool = False) -> List[etree._Element]:
     """
     Evaluates and xpath and creates tag if the result is empty
 
@@ -110,6 +124,7 @@ def eval_xpath_create(xmltree,
     :param occurrences: int or list of int. Which occurence of the parent nodes to create a tag if the tag is missing.
                         By default all nodes are used.
     :param list_return: if True, the returned quantity is always a list even if only one element is in it
+    :param number_nodes: how many identical nodes to create
 
     :returns: list of nodes from the result of the xpath expression
     """
@@ -117,34 +132,35 @@ def eval_xpath_create(xmltree,
 
     check_complex_xpath(xmltree, base_xpath, xpath)
 
-    nodes = eval_xpath(xmltree, xpath, list_return=True)
+    nodes: List[etree._Element] = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
 
     if len(nodes) == 0:
         parent_xpath, tag_name = split_off_tag(base_xpath)
-        complex_parent_xpath, _ = split_off_tag(xpath)
+        complex_parent_xpath, _ = split_off_tag(str(xpath))
         xmltree = xml_create_tag_schema_dict(xmltree,
                                              schema_dict,
                                              complex_parent_xpath,
                                              parent_xpath,
                                              tag_name,
                                              create_parents=create_parents,
+                                             number_nodes=number_nodes,
                                              occurrences=occurrences)
-        nodes = eval_xpath(xmltree, xpath, list_return=True)
+        nodes = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
 
     if len(nodes) == 1 and not list_return:
-        nodes = nodes[0]
+        nodes = nodes[0]  #type:ignore
 
     return nodes
 
 
-def xml_set_attrib_value(xmltree,
-                         schema_dict,
-                         xpath,
-                         base_xpath,
-                         attributename,
-                         attribv,
-                         occurrences=None,
-                         create=False):
+def xml_set_attrib_value(xmltree: Union[etree._Element, etree._ElementTree],
+                         schema_dict: 'fleur_schema.SchemaDict',
+                         xpath: 'etree._xpath',
+                         base_xpath: str,
+                         attributename: str,
+                         attribv: Any,
+                         occurrences: Union[int, Iterable[int]] = None,
+                         create: bool = False) -> Union[etree._Element, etree._ElementTree]:
     """
     Sets an attribute in a xmltree to a given value. By default the attribute will be set
     on all nodes returned for the specified xpath.
@@ -172,25 +188,10 @@ def xml_set_attrib_value(xmltree,
     from masci_tools.util.xml.xml_setters_basic import xml_set_attrib_value_no_create
     from masci_tools.util.xml.converters import convert_to_xml
     from masci_tools.util.xml.common_functions import check_complex_xpath, split_off_tag
+    from masci_tools.io.common_functions import is_sequence
 
     check_complex_xpath(xmltree, base_xpath, xpath)
 
-    if create:
-        nodes = eval_xpath_create(xmltree,
-                                  schema_dict,
-                                  xpath,
-                                  base_xpath,
-                                  create_parents=True,
-                                  occurrences=occurrences,
-                                  list_return=True)
-    else:
-        nodes = eval_xpath(xmltree, xpath, list_return=True)
-
-    if len(nodes) == 0:
-        raise ValueError(f"Could not set attribute '{attributename}' on path '{xpath}' "
-                         'because atleast one subtag is missing. '
-                         'Use create=True to create the subtags')
-
     _, tag_name = split_off_tag(base_xpath)
 
     attribs = schema_dict['tag_info'][base_xpath]['attribs']
@@ -201,11 +202,35 @@ def xml_set_attrib_value(xmltree,
     attributename = attribs.original_case[attributename]
 
     converted_attribv, _ = convert_to_xml(attribv, schema_dict, attributename, text=False)
+    n_nodes = len(converted_attribv) if is_sequence(converted_attribv) else 1
+
+    if create:
+        nodes: List[etree._Element] = eval_xpath_create(xmltree,
+                                                        schema_dict,
+                                                        xpath,
+                                                        base_xpath,
+                                                        create_parents=True,
+                                                        occurrences=occurrences,
+                                                        number_nodes=n_nodes,
+                                                        list_return=True)
+    else:
+        nodes = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
+
+    if len(nodes) == 0:
+        raise ValueError(f"Could not set attribute '{attributename}' on path '{str(xpath)}' "
+                         'because atleast one subtag is missing. '
+                         'Use create=True to create the subtags')
 
     return xml_set_attrib_value_no_create(xmltree, xpath, attributename, converted_attribv, occurrences=occurrences)
 
 
-def xml_set_first_attrib_value(xmltree, schema_dict, xpath, base_xpath, attributename, attribv, create=False):
+def xml_set_first_attrib_value(xmltree: Union[etree._Element, etree._ElementTree],
+                               schema_dict: 'fleur_schema.SchemaDict',
+                               xpath: 'etree._xpath',
+                               base_xpath: str,
+                               attributename: str,
+                               attribv: Any,
+                               create: bool = False) -> Union[etree._Element, etree._ElementTree]:
     """
     Sets the first occurrence attribute in a xmltree to a given value.
     If there are no nodes under the specified xpath a tag can be created with `create=True`.
@@ -238,7 +263,13 @@ def xml_set_first_attrib_value(xmltree, schema_dict, xpath, base_xpath, attribut
                                 occurrences=0)
 
 
-def xml_set_text(xmltree, schema_dict, xpath, base_xpath, text, occurrences=None, create=False):
+def xml_set_text(xmltree: Union[etree._Element, etree._ElementTree],
+                 schema_dict: 'fleur_schema.SchemaDict',
+                 xpath: 'etree._xpath',
+                 base_xpath: str,
+                 text: Any,
+                 occurrences: Union[int, Iterable[int]] = None,
+                 create: bool = False) -> Union[etree._Element, etree._ElementTree]:
     """
     Sets the text on tags in a xmltree to a given value. By default the text will be set
     on all nodes returned for the specified xpath.
@@ -263,32 +294,40 @@ def xml_set_text(xmltree, schema_dict, xpath, base_xpath, text, occurrences=None
     from masci_tools.util.xml.xml_setters_basic import xml_set_text_no_create
     from masci_tools.util.xml.converters import convert_to_xml
     from masci_tools.util.xml.common_functions import check_complex_xpath, split_off_tag
+    from masci_tools.io.common_functions import is_sequence
 
     check_complex_xpath(xmltree, base_xpath, xpath)
 
-    if create:
-        nodes = eval_xpath_create(xmltree,
-                                  schema_dict,
-                                  xpath,
-                                  base_xpath,
-                                  create_parents=True,
-                                  occurrences=occurrences,
-                                  list_return=True)
-    else:
-        nodes = eval_xpath(xmltree, xpath, list_return=True)
-
-    if len(nodes) == 0:
-        raise ValueError(f"Could not set text on path '{xpath}' because atleast one subtag is missing. "
-                         'Use create=True to create the subtags')
-
     _, tag_name = split_off_tag(base_xpath)
 
     converted_text, _ = convert_to_xml(text, schema_dict, tag_name, text=True)
+    n_nodes = len(converted_text) if is_sequence(converted_text) else 1
+
+    if create:
+        nodes: List[etree._Element] = eval_xpath_create(xmltree,
+                                                        schema_dict,
+                                                        xpath,
+                                                        base_xpath,
+                                                        create_parents=True,
+                                                        occurrences=occurrences,
+                                                        number_nodes=n_nodes,
+                                                        list_return=True)
+    else:
+        nodes = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
+
+    if len(nodes) == 0:
+        raise ValueError(f"Could not set text on path '{str(xpath)}' because atleast one subtag is missing. "
+                         'Use create=True to create the subtags')
 
     return xml_set_text_no_create(xmltree, xpath, converted_text, occurrences=occurrences)
 
 
-def xml_set_first_text(xmltree, schema_dict, xpath, base_xpath, text, create=False):
+def xml_set_first_text(xmltree: Union[etree._Element, etree._ElementTree],
+                       schema_dict: 'fleur_schema.SchemaDict',
+                       xpath: 'etree._xpath',
+                       base_xpath: str,
+                       text: Any,
+                       create: bool = False) -> Union[etree._Element, etree._ElementTree]:
     """
     Sets the text on the first occurrence of a tag in a xmltree to a given value.
     If there are no nodes under the specified xpath a tag can be created with `create=True`.
@@ -311,14 +350,15 @@ def xml_set_first_text(xmltree, schema_dict, xpath, base_xpath, text, create=Fal
     return xml_set_text(xmltree, schema_dict, xpath, base_xpath, text, create=create, occurrences=0)
 
 
-def xml_add_number_to_attrib(xmltree,
-                             schema_dict,
-                             xpath,
-                             base_xpath,
-                             attributename,
-                             add_number,
-                             mode='abs',
-                             occurrences=None):
+def xml_add_number_to_attrib(
+        xmltree: Union[etree._Element, etree._ElementTree],
+        schema_dict: 'fleur_schema.SchemaDict',
+        xpath: 'etree._xpath',
+        base_xpath: str,
+        attributename: str,
+        add_number: Any,
+        mode: Literal['abs', 'rel'] = 'abs',
+        occurrences: Union[int, Iterable[int]] = None) -> Union[etree._Element, etree._ElementTree]:
     """
     Adds a given number to the attribute value in a xmltree. By default the attribute will be shifted
     on all nodes returned for the specified xpath.
@@ -355,7 +395,7 @@ def xml_add_number_to_attrib(xmltree,
     possible_types = schema_dict['attrib_types'][attributename]
 
     if not etree.iselement(xmltree):
-        constants = read_constants(xmltree.getroot(), schema_dict)
+        constants = read_constants(xmltree.getroot(), schema_dict)  #type:ignore
     else:
         constants = read_constants(xmltree, schema_dict)
 
@@ -373,28 +413,29 @@ def xml_add_number_to_attrib(xmltree,
             f'Allowed attributes are: {attribs.original_case.values()}')
     attributename = attribs.original_case[attributename]
 
-    if not xpath.endswith(f'/@{attributename}'):
-        xpath = '/@'.join([xpath, attributename])
+    if not str(xpath).endswith(f'/@{attributename}'):
+        xpath = '/@'.join([str(xpath), attributename])
 
-    stringattribute = eval_xpath(xmltree, xpath, list_return=True)
+    stringattribute: List[str] = eval_xpath(xmltree, xpath, list_return=True)  #type:ignore
 
-    tag_xpath, attributename = split_off_attrib(xpath)
+    tag_xpath, attributename = split_off_attrib(str(xpath))
 
     if len(stringattribute) == 0:
         raise ValueError(f"No attribute values found for '{attributename}'. Cannot add number")
 
-    attribvalues, _ = convert_from_xml(stringattribute,
-                                       schema_dict,
-                                       attributename,
-                                       text=False,
-                                       constants=constants,
-                                       list_return=True)
+    res: Tuple[List[Union[int, float]], bool] = convert_from_xml(stringattribute,
+                                                                 schema_dict,
+                                                                 attributename,
+                                                                 text=False,
+                                                                 constants=constants,
+                                                                 list_return=True)  #type:ignore
+    attribvalues, _ = res
 
     if occurrences is not None:
         if not is_sequence(occurrences):
-            occurrences = [occurrences]
+            occurrences = [occurrences]  #type:ignore
         try:
-            attribvalues = [attribvalues[occ] for occ in occurrences]
+            attribvalues = [attribvalues[occ] for occ in cast(Iterable[int], occurrences)]
         except IndexError as exc:
             raise ValueError('Wrong value for occurrences') from exc
 
@@ -421,7 +462,13 @@ def xml_add_number_to_attrib(xmltree,
     return xmltree
 
 
-def xml_add_number_to_first_attrib(xmltree, schema_dict, xpath, base_xpath, attributename, add_number, mode='abs'):
+def xml_add_number_to_first_attrib(xmltree: Union[etree._Element, etree._ElementTree],
+                                   schema_dict: 'fleur_schema.SchemaDict',
+                                   xpath: 'etree._xpath',
+                                   base_xpath: str,
+                                   attributename: str,
+                                   add_number: Any,
+                                   mode: Literal['abs', 'rel'] = 'abs') -> Union[etree._Element, etree._ElementTree]:
     """
     Adds a given number to the first occurrence of a attribute value in a xmltree.
     If there are no nodes under the specified xpath an error is raised
@@ -452,7 +499,13 @@ def xml_add_number_to_first_attrib(xmltree, schema_dict, xpath, base_xpath, attr
                                     occurrences=0)
 
 
-def xml_set_simple_tag(xmltree, schema_dict, xpath, base_xpath, tag_name, changes, create_parents=False):
+def xml_set_simple_tag(xmltree: Union[etree._Element, etree._ElementTree],
+                       schema_dict: 'fleur_schema.SchemaDict',
+                       xpath: 'etree._xpath',
+                       base_xpath: str,
+                       tag_name: str,
+                       changes: Union[List[Dict[str, Any]], Dict[str, Any]],
+                       create_parents: bool = False) -> Union[etree._Element, etree._ElementTree]:
     """
     Sets one or multiple `simple` tag(s) in an xmltree. A simple tag can only hold attributes and has no
     subtags.
@@ -478,7 +531,7 @@ def xml_set_simple_tag(xmltree, schema_dict, xpath, base_xpath, tag_name, change
 
     tag_info = schema_dict['tag_info'][base_xpath]
 
-    tag_xpath = f'{xpath}/{tag_name}'
+    tag_xpath = f'{str(xpath)}/{tag_name}'
     tag_base_xpath = f'{base_xpath}/{tag_name}'
 
     if tag_name in tag_info['several']:
@@ -486,17 +539,23 @@ def xml_set_simple_tag(xmltree, schema_dict, xpath, base_xpath, tag_name, change
         if isinstance(changes, dict):
             changes = [changes]
 
-        # policy: we DELETE all existing tags, and create new ones from the given parameters.
-        xml_delete_tag(xmltree, tag_xpath)
+        if len(eval_xpath(xmltree, tag_xpath, list_return=True)) > 0:  #type:ignore
+            # policy: we DELETE all existing tags, and create new ones from the given parameters.
+            xml_delete_tag(xmltree, tag_xpath)
 
-        for indx in range(0, len(changes)):
-            xml_create_tag_schema_dict(xmltree, schema_dict, xpath, base_xpath, tag_name, create_parents=create_parents)
+        xml_create_tag_schema_dict(xmltree,
+                                   schema_dict,
+                                   xpath,
+                                   base_xpath,
+                                   tag_name,
+                                   create_parents=create_parents,
+                                   number_nodes=len(changes))
 
         for indx, change in enumerate(changes):
             for attrib, value in change.items():
                 occurrences = [
                     k * len(changes) + indx
-                    for k in range(len(eval_xpath(xmltree, tag_xpath, list_return=True)) // len(changes))
+                    for k in range(len(eval_xpath(xmltree, tag_xpath, list_return=True)) // len(changes))  #type:ignore
                 ]
                 xml_set_attrib_value(xmltree,
                                      schema_dict,
@@ -518,7 +577,12 @@ def xml_set_simple_tag(xmltree, schema_dict, xpath, base_xpath, tag_name, change
     return xmltree
 
 
-def xml_set_complex_tag(xmltree, schema_dict, xpath, base_xpath, attributedict, create=False):
+def xml_set_complex_tag(xmltree: Union[etree._Element, etree._ElementTree],
+                        schema_dict: 'fleur_schema.SchemaDict',
+                        xpath: 'etree._xpath',
+                        base_xpath: str,
+                        attributedict: Dict[str, Any],
+                        create: bool = False) -> Union[etree._Element, etree._ElementTree]:
     """
     Recursive Function to correctly set tags/attributes for a given tag.
     Goes through the attributedict and decides based on the schema_dict, how the corresponding
@@ -565,7 +629,7 @@ def xml_set_complex_tag(xmltree, schema_dict, xpath, base_xpath, attributedict,
 
         key = (tag_info['complex'] | tag_info['simple'] | tag_info['attribs']).original_case[key]
 
-        sub_xpath = f'{xpath}/{key}'
+        sub_xpath = f'{str(xpath)}/{key}'
         sub_base_xpath = f'{base_xpath}/{key}'
         if key in tag_info['attribs']:
             xml_set_attrib_value(xmltree, schema_dict, xpath, base_xpath, key, val, create=create)
@@ -584,8 +648,9 @@ def xml_set_complex_tag(xmltree, schema_dict, xpath, base_xpath, attributedict,
             xmltree = xml_set_complex_tag(xmltree, schema_dict, sub_xpath, sub_base_xpath, val, create=create)
 
         else:
-            # policy: we DELETE all existing tags, and create new ones from the given parameters.
-            xml_delete_tag(xmltree, sub_xpath)
+            if len(eval_xpath(xmltree, sub_xpath, list_return=True)) > 0:  #type:ignore
+                # policy: we DELETE all existing tags, and create new ones from the given parameters.
+                xml_delete_tag(xmltree, sub_xpath)
 
             if isinstance(val, dict):
                 val = [val]
@@ -594,7 +659,7 @@ def xml_set_complex_tag(xmltree, schema_dict, xpath, base_xpath, attributedict,
                 xml_create_tag_schema_dict(xmltree, schema_dict, xpath, base_xpath, key, create_parents=create)
 
             for indx, tagdict in enumerate(val):
-                for k in range(len(eval_xpath(xmltree, sub_xpath, list_return=True)) // len(val)):
+                for k in range(len(eval_xpath(xmltree, sub_xpath, list_return=True)) // len(val)):  #type:ignore
                     current_elem_xpath = f'{sub_xpath}[{k*len(val)+indx+1}]'
                     xmltree = xml_set_complex_tag(xmltree,
                                                   schema_dict,
